from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class RedShiftSSL(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Redshift uses SSL"
        id = "CKV_AWS_105"
        supported_resources = ['aws_redshift_parameter_group']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf):
        if 'parameter' not in conf:
            return CheckResult.FAILED
        self.evaluated_keys = ['parameter']
        for idx, elem in enumerate(conf["parameter"]):
            if isinstance(elem, dict) and elem["name"][0] == "require_ssl" and elem["value"] == [True]:
                self.evaluated_keys = [f'parameter/[{idx}]/name', f'parameter/[{idx}]/value']
                return CheckResult.PASSED

        # no matching params
        return CheckResult.FAILED


check = RedShiftSSL()



from __future__ import annotations
import logging
import json
import subprocess  # nosec
from checkov.common.util.http_utils import request_wrapper
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from typing import Any

logger = logging.getLogger(__name__)


def report_contributor_metrics(repository: str, bc_integration: BcPlatformIntegration) -> None:  # ignore: type
    logging.debug(f"Attempting to get log history for repository {repository}")
    request_body = parse_gitlog(repository)
    if request_body:
        response = request_wrapper(
            "POST", f"{bc_integration.api_url}/api/v1/contributors/report",
            headers=bc_integration.get_default_headers("POST"), data=json.dumps(request_body)
        )
        if response.status_code < 300:
            logging.info(f"Successfully uploaded contributor metrics with status: {response.status_code}")
        else:
            logging.info(f"Failed to upload contributor metrics with: {response.status_code} - {response.reason}")


def parse_gitlog(repository: str) -> dict[str, Any] | None:
    process = subprocess.Popen(['git', 'shortlog', '-ne', '--all', '--since', '"90 days ago"', '--pretty=commit-%ct', '--reverse'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
    out, err = process.communicate()
    if err:
        logger.info(f"Failed to collect contributor metrics due to: {err}")     # type: ignore
        return None
    # split per contributor
    list_of_contributors = out.decode('utf-8').split('\n\n')
    return {"repository": repository,
            "contributors": list(map(lambda contributor: process_contributor(contributor),
                                     list(filter(lambda x: x, list_of_contributors))
                                     ))
            }


def process_contributor(contributor: str) -> str:
    splittedList = contributor.split('\n')
    user = splittedList[0]
    commit = splittedList[1]
    return user[:user.find('(')] + commit[commit.find('-') + 1:]



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing_extensions import Literal

DOCKERFILE_STARTLINE: Literal["startline"] = "startline"
DOCKERFILE_ENDLINE: Literal["endline"] = "endline"



import re
from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.provider.base_check import BaseProviderCheck
from checkov.common.models.consts import access_key_pattern, secret_key_pattern


class NCPCredentials(BaseProviderCheck):
    def __init__(self) -> None:
        name = "Ensure no hard coded NCP access key and secret key exists in provider"
        id = "CKV_NCP_17"
        supported_provider = ("ncloud",)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
        see: https://registry.terraform.io/providers/NaverCloudPlatform/ncloud/latest/docs
        """
        result = CheckResult.PASSED
        if self.secret_found(conf, "access_key", access_key_pattern):
            result = CheckResult.FAILED
        if self.secret_found(conf, "secret_key", secret_key_pattern):
            result = CheckResult.FAILED
        return result

    def secret_found(self, conf: Dict[str, List[Any]], field: str, pattern: str) -> bool:
        if field in conf.keys():
            value = conf[field][0]
            if isinstance(value, str) and re.match(pattern, value) is not None:
                conf[f'{self.id}_secret_{field}'] = value
                return True
        return False


check = NCPCredentials()




from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ServerEncryptionVPC(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Server instance is encrypted."
        id = "CKV_NCP_6"
        supported_resources = ('ncloud_server',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
    
    def get_inspected_key(self):
        return "is_encrypted_base_block_storage_volume"


check = ServerEncryptionVPC()


from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class AccessControlGroupOutboundRule(BaseResourceCheck):
    def __init__(self):
        name = "Ensure no security group rules allow outbound traffic to 0.0.0.0/0"
        id = "CKV_NCP_3"
        supported_resources = ('ncloud_access_control_group_rule',)

        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories,
                         supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        for outbound in conf.get('outbound', []):
            ip = outbound.get('ip_block')
            if ip == ['0.0.0.0/0'] or ip == ['::/0']:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = AccessControlGroupOutboundRule()